(****************************************************************************)
(*                 The Calculus of Inductive Constructions                  *)
(*                                                                          *)
(*                                Projet Coq                                *)
(*                                                                          *)
(*                     INRIA                        ENS-CNRS                *)
(*              Rocquencourt                        Lyon                    *)
(*                                                                          *)
(*                                 Coq V6.1                                 *)
(*                               Oct 1st 1996                               *)
(*                                                                          *)
(****************************************************************************)
(*                                  Bool.v                                  *)
(****************************************************************************)


(************)
(* Booleans *)
(************)

(* Inductive bool : Set := true : bool | false : bool (from Prelude) *)

Definition Is_true := [b:bool](Cases b of
                                 true  => True
                               | false => False
                               end).

(******************)
(* Discrimination *)
(******************)

Lemma diff_true_false : ~true=false.
Goal.
Unfold not; Intro contr; Change (Is_true false).
Elim contr; Simpl; Trivial.
Save.
Hint diff_true_false.

Lemma diff_false_true : ~false=true.
Goal.
(Red; Intros H; Apply diff_true_false).
Symmetry.
Assumption.
Qed.
Hint diff_false_true.

Lemma eq_true_false_abs : (b:bool)(b=true)->(b=false)->False.
Intros b H; Rewrite H; Auto.
Save.
Hint eq_true_false_abs.

Lemma not_true_is_false : (b:bool)~b=true->b=false.
Destruct b.
Intros.
(Red in H; Elim H).
Reflexivity.
Intros abs.
Reflexivity.
Qed.

Lemma not_false_is_true : (b:bool)~b=false->b=true.
Destruct b.
Intros.
Reflexivity.
Intro H; Red in H; Elim H.
Reflexivity.
Qed.

(************)
(* Equality *)
(************)

Definition eqb : bool->bool->bool :=
 [b1,b2:bool]
    Cases b1 b2 of
      true  true  => true
    | true  false => false
    | false true  => false
    | false false => true
    end.

Lemma eqb_refl : (x:bool)(Is_true (eqb x x)).
Destruct x; Simpl; Auto.
Save.

Lemma eqb_eq : (x,y:bool)(Is_true (eqb x y))->x=y.
Destruct x; Destruct y; Simpl; Tauto.
Save.

Lemma Is_true_eq_true : (x:bool) (Is_true x) -> x=true.
Destruct x; Simpl; Tauto.
Save.
 
Lemma Is_true_eq_true2 : (x:bool)  x=true -> (Is_true x).
Destruct x; Simpl; Auto.
Save.

Lemma eqb_subst : 
  (P:bool->Prop)(b1,b2:bool)(eqb b1 b2)=true->(P b1)->(P b2).
Unfold eqb .
Intros P b1.
Intros b2.
Case b1.
Case b2.
Trivial.
Intros H.
Inversion_clear H.
Case b2.
Intros H.
Inversion_clear H.
Trivial.
Qed.
 
Lemma eqb_reflx : (b:bool)(eqb b b)=true.
Intro b.
Case b.
Trivial.
Trivial.
Qed.

Lemma eqb_prop : (a,b:bool)(eqb a b)=true -> a=b.
Destruct a; Destruct b; Simpl; Intro;
  Discriminate H Orelse Reflexivity.
Save.

(************)
(* Negation *)
(************)

Definition negb := [b:bool](Cases b of
                             true  => false
                           | false => true
                           end).

Lemma negb_intro : (b:bool)(negb (negb b))=b.
Goal.
Induction b; Simpl; Auto.
Save.

Lemma negb_elim : (b:bool)b=(negb (negb b)).
Goal.
Induction b; Simpl; Trivial.
Save.
       
Lemma negb_sym : (b,b':bool)(b'=(negb b))->(b=(negb b')).
Goal.
Induction b; Induction b'; Intros; Simpl; Trivial.
Save.

Lemma no_fixpoint_negb : (b:bool)~(negb b)=b.
Goal.
Induction b; Simpl; Unfold not; Intro; Apply diff_true_false; Auto.
Save.

Lemma eqb_negb1 : (b:bool)(eqb (negb b) b)=false.
Destruct b.
Trivial.
Trivial.
Qed.
 
Lemma eqb_negb2 : (b:bool)(eqb b (negb b))=false.
Destruct b.
Trivial.
Trivial.
Qed.

(***********************)
(* Logical combinators *)
(***********************)
 
Definition ifb : bool -> bool -> bool -> bool
     := [b1,b2,b3:bool](Cases b1 of true => b2 | false => b3 end).

Definition andb : bool -> bool -> bool
     := [b1,b2:bool](ifb b1 b2 false).

Definition orb : bool -> bool -> bool
     := [b1,b2:bool](ifb b1 true b2).

Definition implb : bool -> bool -> bool
     := [b1,b2:bool](ifb b1 b2 true).

Definition xorb :  bool -> bool -> bool
     := [b1,b2:bool]
    Cases b1 b2 of
      true  true  => false
    | true  false => true
    | false true  => true
    | false false => false
    end.

(*************************)
(* A few lemmas about Or *)
(*************************)


Lemma orb_prop : 
  (a,b:bool)(orb a b)=true -> (a = true)\/(b = true).
Induction a; Induction b; Simpl; Try (Intro H;Discriminate H); Auto.
Qed.

Lemma orb_prop2 : 
  (a,b:bool)(Is_true (orb a b)) -> (Is_true a)\/(Is_true b).
Induction a; Induction b; Simpl; Try (Intro H;Discriminate H); Auto.
Qed.

Lemma orb_true_intro 
    : (b1,b2:bool)(b1=true)\/(b2=true)->(orb b1 b2)=true.
Destruct b1; Auto.
Destruct 1; Intros.
Elim diff_true_false; Auto.
Rewrite H0; Trivial.
Save.
Hint orb_true_intro.

Lemma orb_b_true : (b:bool)(orb b true)=true.
Auto.
Save.
Hint orb_b_true.

Lemma orb_true_b : (b:bool)(orb true b)=true.
Trivial.
Save.

Lemma orb_true_elim : (b1,b2:bool)(orb b1 b2)=true -> {b1=true}+{b2=true}.
Destruct b1; [Auto | Destruct b2; Auto].
Save.
Lemma orb_false_intro 
    : (b1,b2:bool)(b1=false)->(b2=false)->(orb b1 b2)=false.
Intros b1 b2 H1 H2; Rewrite H1; Rewrite H2; Trivial.
Save.
Hint orb_false_intro.

Lemma orb_b_false : (b:bool)(orb b false)=b.
Destruct b; Trivial.
Save.
Hint orb_b_false.


Lemma orb_false_elim : 
    (b1,b2:bool)(orb b1 b2)=false -> (b1=false)/\(b2=false).
Destruct b1.
Intros; Elim diff_true_false; Auto.
Destruct b2.
Intros; Elim diff_true_false; Auto.
Auto.
Save.

(**************************)
(* A few lemmas about And *)
(**************************)

Lemma andb_prop : 
  (a,b:bool)(andb a b) = true -> (a = true)/\(b = true).

Induction a; Induction b; Simpl; Try (Intro H;Discriminate H);Auto.
Qed.

Lemma andb_prop2 : 
  (a,b:bool)(Is_true (andb a b)) -> (Is_true a)/\(Is_true b).
Induction a; Induction b; Simpl; Try (Intro H;Discriminate H); Auto.
Qed.

Lemma andb_true_intro 
    : (b1,b2:bool)(b1=true)/\(b2=true)->(andb b1 b2)=true.
Destruct b1; Destruct b2; Simpl; Tauto Orelse Auto.
Save.
Hint andb_true_intro.

Lemma andb_true_intro2 
    : (b1,b2:bool)(b1=true)->(b2=true)->(andb b1 b2)=true.
Intros b1 b2 H1 H2; Rewrite H1; Rewrite H2; Trivial.
Save.
Hint andb_true_intro2.

Lemma andb_false_intro1 
    : (b1,b2:bool)(b1=false)->(andb b1 b2)=false.
Destruct b1; Destruct b2; Simpl; Tauto Orelse Auto.
Save.

Lemma andb_false_intro2 
    : (b1,b2:bool)(b2=false)->(andb b1 b2)=false.
Destruct b1; Destruct b2; Simpl; Tauto Orelse Auto.
Save.

Lemma andb_b_false : (b:bool)(andb b false)=false.
Destruct b; Auto.
Save.
Hint andb_b_false.

Lemma andb_false_b : (b:bool)(andb false b)=false.
Trivial.
Save.

Lemma andb_false_elim : 
    (b1,b2:bool)(andb b1 b2)=false -> {b1=false}+{b2=false}.
Destruct b1; Destruct b2; Simpl; Auto.
Save.


(* $Id: Bool.v,v 1.10 1998/01/08 14:46:39 loiseleu Exp $ *)

