(****************************************************************************)
(*                 The Calculus of Inductive Constructions                  *)
(*                                                                          *)
(*                                Projet Coq                                *)
(*                                                                          *)
(*                     INRIA                        ENS-CNRS                *)
(*              Rocquencourt                        Lyon                    *)
(*                                                                          *)
(*                                 Coq V6.1                                 *)
(*                               Oct 1st 1996                               *)
(*                                                                          *)
(****************************************************************************)
(*                                 Logic.v                                  *)
(****************************************************************************)

Require Export LogicSyntax.
Require Export Datatypes.

(* [True] is the always true proposition *)
Inductive True : Prop := I : True. 

(* [False] is the always false proposition *)
Inductive False : Prop := . 

(* [not A], written [~A], is the negation of [A] *)
Definition not := [A:Prop]
 A->False.

Section Conjunction.

  (* [and A B], written [A /\ B], is the conjunction of [A] and [B] *)
  (* [conj A B p q], written [<p,q>] is a proof of [A /\ B] as soon as 
     [p] is a proof of [A] and [q] a proof of [B] *)
  (* [proj1] and [proj2] are first and second projections of a conjunction *)

  Inductive and [A,B:Prop] : Prop := conj : A -> B -> A/\B.

  Variables A,B : Prop.

  Theorem proj1 : A/\B -> A.
  Proof.
  Induction 1; Trivial.
  Qed.

  Theorem proj2 : A/\B -> B.
  Proof.
  Induction 1; Trivial.
  Qed.

End Conjunction.


Section Disjunction.

  (* [or A B], written [A \/ B], is the disjunction of [A] and [B] *)

  Inductive or [A,B:Prop] : Prop
      := or_introl : A -> A\/B 
       | or_intror : B -> A\/B.

End Disjunction.

Section Equivalence.

  (* [iff A B], written [A <-> B], expresses the equivalence of [A] and [B] *)

  Definition iff := [P,Q:Prop] (P->Q) /\ (Q->P).

End Equivalence.

(* [(IF P Q R)], or more suggestively [(either P and_then Q or_else R)],
   denotes either [P] and [Q], or [~P] and [Q] *)
Definition IF := [P,Q,R:Prop] (P/\Q) \/ (~P/\R).


Section First_order_quantifiers.

  (* [(ex A P)], or simply [(EX x | P(x))], expresses the existence of an 
     [x] of type [A] which satisfies the predicate [P] ([A] is of type 
     [Set]). This is existential quantification. *)

  (* [(ex2 A P Q)], or simply [(EX x | P(x) & Q(x))], expresses the
     existence of an [x] of type [A] which satisfies both the predicates
     [P] and [Q] *)

  (* Universal quantification (especially first-order one) is normally 
     written [(x:A)(P x)]. For duality with existential quantification, the 
     construction [(all A P)], or simply [(All P)], is provided as an 
     abbreviation of [(x:A)(P x)] *) 

  Inductive ex [A:Set;P:A->Prop] : Prop 
      := ex_intro : (x:A)(P x)->(ex A P).

  Inductive ex2 [A:Set;P,Q:A->Prop] : Prop
      := ex_intro2 : (x:A)(P x)->(Q x)->(ex2 A P Q).

  Definition all := [A:Set][P:A->Prop](x:A)(P x). 

End First_order_quantifiers.

Section Equality.

  (* [(eq A x y)], or simply [x=y], expresses the (Leibniz') equality
     of [x] and [y]. Both [x] and [y] must belong to the same type [A].
     The definition is inductive and states the reflexivity of the equality.
     The others properties (symmetry, transitivity, replacement of 
     equals) are proved below *)

  Inductive eq [A:Set;x:A] : A->Prop
         := refl_equal : (eq A x x).

End Equality.

Hint I conj or_introl or_intror refl_equal.

Section Logic_lemmas.

  Theorem absurd : (A:Prop)(C:Prop) A -> ~A -> C.
  Proof.
    Unfold not; Intros A C h1 h2.
    Elim (h2 h1).
  Qed.

  Section equality.
    Variable A,B : Set.
    Variable f   : A->B.
    Variable x,y,z : A.

    Theorem sym_eq : x=y -> y=x.
    Proof.
     Induction 1; Trivial.
    Qed.

    Theorem trans_eq : x=y -> y=z -> x=z.
    Proof.
     Induction 2; Trivial.
    Qed.

    Theorem f_equal : x=y -> (f x)=(f y).
    Proof.
     Induction 1; Trivial.
    Qed.

    Theorem sym_not_eq : ~(x=y) -> ~(y=x).
    Proof.
     Red; Intros h1 h2; Apply h1; Elim h2; Trivial.
    Qed.

    Definition sym_equal     := sym_eq.
    Definition sym_not_equal := sym_not_eq.
    Definition trans_equal   := trans_eq.

  End equality.

  Theorem eq_rect: (A:Set)(x:A)(P:A->Type)(P x)->(y:A)x=y->(P y).
  Proof.
   Intros.
   Cut (identity A x y).
   Destruct 1; Auto.
   Elim H; Auto.
  Qed.

  Definition eq_ind_r : (A:Set)(x:A)(P:A->Prop)(P x)-> (y:A) y=x -> (P y).
   Intros A x P H y H0; Case sym_eq with 1:= H0; Trivial.
  Defined.

  Definition eq_rec_r : (A:Set)(x:A)(P:A->Set)(P x)->(y:A)y=x->(P y).
    Intros A x P H y H0; Case sym_eq with 1:= H0; Trivial.
  Defined.

  Definition eq_rect_r : (A:Set)(x:A)(P:A->Type)(P x)->(y:A)y=x->(P y).
    Intros A x P H y H0; Elim sym_eq with 1:= H0; Trivial.
  Defined.
End Logic_lemmas.

Immediate sym_eq sym_not_eq.


Syntactic Definition Ex := ex | 1.
Syntactic Definition Ex2 := ex2 | 1.
Syntactic Definition All := all | 1.

(* Pretty-printing of things in Logic.v *)

Syntax constr
  level 1:
    equal [<<(eq $_  $t1  $t2)>>] -> [ [<hov 0> $t1:E [0 1]  "=" $t2:E ] ]
  | conj [<<(conj $t1 $t2 $t3 $t4)>>]
      -> [ [<hov 1> [<hov 1> "<" $t1:L "," [0 0] $t2:L ">" ]
                    [<hov 1> "{" $t3:L "," [0 0] $t4:L "}"] ] ]
  | if [<<(IF $c $t $e)>>]
      -> [ [<hov 0> "either" [1 1] $c:E
                [<hov 0> [1 1] "and_then" [1 1] $t:E ]
                [<hov 0> [1 1] "or_else" [1 1] $e:E ]] ]
  ;

  level 2:
    not [<<(not $t1)>>] -> [ [<hov 0> "~" $t1:E ] ]
  ;

  level 6:
    and [<<(and $t1 $t2)>>] -> [ [<hov 0> $t1:L [0 0] "/\\" $t2:E ] ]
  ;

  level 7:
    or [<<(or $t1 $t2)>>] -> [ [<hov 0> $t1:L [0 0]  "\\/" $t2:E ] ]
  ;

  level 8:
    iff [<<(iff $t1 $t2)>>] -> [ [<hov 0> $t1:L [0 0] "<->" $t2:E ] ]
  ;

  level 10:
    all_pred [<<(all $_ $p)>>] -> [ [<hov 4> "All " $p:L ] ]
  | all_imp [<<(all $_ [$x:$_]$t)>>]
       -> [ [<hov 4> "ALL " $x " |" [1 0] $t:L ] ]

  | ex_pred [<<(ex $_ $p)>>] -> [ [<hov 0> "Ex " $p:L ] ]
  | ex [<<(ex $_ [$x1:$_]$P1)>>] 
       -> [ [<hov 3> "EX " $x1 " |" [1 0] $P1:L ] ]

  | ex2_pred [<<(ex2 $_ $p1 $p2)>>]
       -> [ [<hov 4> "Ex2 " $p1:L [1 0] $p2:L ] ]
  | ex2 [<<(ex2 $_ [$x1:$_]$P1 [$x1:$_]$P2)>>] 
       -> [ [<hov 3> "EX " $x1 " |" [1 0] $P1:L [1 0] "& " $P2:L] ].


(* $Id: Logic.v,v 1.12 1997/12/03 16:02:05 barras Exp $ *)





