(****************************************************************************)
(*                 The Calculus of Inductive Constructions                  *)
(*                                                                          *)
(*                                Projet Coq                                *)
(*                                                                          *)
(*                     INRIA                        ENS-CNRS                *)
(*              Rocquencourt                        Lyon                    *)
(*                                                                          *)
(*                                 Coq V6.1                                 *)
(*                               Oct 1st 1996                               *)
(*                                                                          *)
(****************************************************************************)
(*                                 Specif.v                                 *)
(****************************************************************************)

(**************************************************************)
(* Basic specifications : Sets containing logical information *)
(**************************************************************)

Require Export Logic.
Require Export SpecifSyntax.

Section Subsets.

  (* [(sig A P)], or more suggestively [{x:A | (P x)}], denotes the subset 
     of elements of the Set [A] which satisfy the predicate [P].
     Similarly [(sig2 A P Q)], or [{x:A | (P x) & (Q x)}], denotes the subset 
     of elements of the Set [A] which satisfy both [P] and [Q]. *)

  Inductive sig [A:Set;P:A->Prop] : Set
      := exist : (x:A)(P x) -> (sig A P).

  Inductive sig2 [A:Set;P,Q:A->Prop] : Set
      := exist2 : (x:A)(P x) -> (Q x) -> (sig2 A P Q).

  (* [(sigS A P)], or more suggestively [{x:A & (P x)}], is a subtle variant
     of subset where [P] is now of type [Set].
     Similarly for [(sigS2 A P Q)], also written [{x:A & (P x) & (Q x)}]. *)
     
  Inductive sigS [A:Set;P:A->Set] : Set
      := existS : (x:A)(P x) -> (sigS A P).

  Inductive sigS2 [A:Set;P,Q:A->Set] : Set
      := existS2 : (x:A)(P x) -> (Q x) -> (sigS2 A P Q).


End Subsets.


(***********************)
(* Projections of sig *)
(***********************)

Section Subset_projections.

  Variable A:Set.
  Variable P:A->Prop.

  Definition proj1_sig :=
   [e:(sig A P)]Cases e of (exist a b) => a  end.

  Definition proj2_sig :=
   [e:(sig A P)]
     <[e:(sig A P)](P (proj1_sig e))>Cases e of (exist a b) => b  end.

End Subset_projections.


(***********************)
(* Projections of sigS *)
(***********************)

Section Projections.

  Variable A:Set.
  Variable P:A->Set.

  (* An element [y] of a subset [[{x:A & (P x)}] is the pair of an [a] of 
     type [A] and of a proof [h] that [a] satisfies [P].
     Then [(projS1 y)] is the witness [a]
     and [(projS2 y)] is the proof of [(P a)] *)

  Definition projS1 (* : (A:Set)(P:A->Set)(sigS A P) -> A *)
           := [x:(sigS A P)]Cases x of (existS a _) => a end.
  Definition projS2 (* : (A:Set)(P:A->Set)(H:(sigS A P))(P (projS1 A P H)) *)
           := [x:(sigS A P)]<[x:(sigS A P)](P (projS1 x))> 
                  Cases x of (existS _ h) => h end.

End Projections.

Syntactic Definition ProjS1 := (projS1 ? ?).
Syntactic Definition ProjS2 := (projS2 ? ?).


Section Extended_booleans.

  (* Syntax sumbool "{_}+{_}". *)
  Inductive sumbool [A,B:Prop] : Set
      := left  : A -> ({A}+{B}) 
       | right : B -> ({A}+{B}).

  (* Syntax sumor "_+{_}". *)
  Inductive sumor [A:Set;B:Prop] : Set
      := inleft  : A -> (A+{B}) 
       | inright : B -> (A+{B}).

End Extended_booleans.


(**********)
(* Choice *)
(**********)

Section Choice_lemmas.

  (* The following lemmas state various forms of the axiom of choice *)

  Variables S,S':Set.
  Variable R:S->S'->Prop.
  Variable R':S->S'->Set.
  Variables R1,R2 :S->Prop.

  Lemma Choice : ((x:S){y:S'|(R x y)}) -> {f:S->S'|(z:S)(R z (f z))}.
  Proof.
   Intro H.
   Exists [z:S]Cases (H z) of (exist y _) => y end.
   Intro z; Elim (H z); Trivial.
  Qed.

  Lemma Choice2 : ((x:S){y:S' & (R' x y)}) -> {f:S->S' & (z:S)(R' z (f z))}.
  Proof.
    Intro H.
    Exists [z:S]Cases (H z) of (existS y _) => y end.
    Intro z; Elim (H z); Trivial.
  Qed.

  Lemma bool_choice : 
    ((x:S){(R1 x)}+{(R2 x)}) ->
    {f:S->bool | (x:S)( ((f x)=true  /\ (R1 x)) 
                     \/ ((f x)=false /\ (R2 x)))}.
  Proof.
    Intro H.
    Exists [z:S]Cases (H z) of (left _) => true | (right _) => false end.
    Intro z; Elim (H z); Auto.
  Qed.

End Choice_lemmas.

Section Exceptions.

  (* A result of type [(Exc A)] is either a normal value of type [A] or 
     an [error]. *) 

  Inductive Exc [A:Set] : Set := value : A->(Exc A) 
                               | error : (Exc A).

End Exceptions.

Syntactic Definition Error := (error ?).
Syntactic Definition Value := (value ?).


(*******************************)
(* Self realizing propositions *)
(*******************************)

Axiom False_rec : (P:Set)False->P.

Lemma False_rect: (C:Type)False->C.
Proof.
  Intros.
  Cut Empty_set.
  Destruct 1.
  Elim H.
Qed.


Definition except := False_rec. (* for compatibility with previous versions *)

Syntactic Definition Except := (except ?).

Theorem absurd_set : (A:Prop)(C:Set)A->(~A)->C.
Proof.
  Intros A C h1 h2.
  Apply False_rec.
  Apply (h2 h1).
Qed.

Theorem and_rec : (A,B:Prop)(C:Set)(A->B->C)->(A/\B)->C.
Proof.
  Intros A B C F AB; Apply F; Elim AB; Auto.
Qed.

(** is now a theorem
Axiom eq_rec : (A:Set)(a:A)(P:A->Set)(P a)->(b:A) a=b -> (P b).
**)

Hint left right inleft inright.


(*********************************)
(* Sigma Type at Type level sigT *)
(*********************************)

Inductive sigT [A:Type;P:A->Type] : Type
    := existT : (x:A)(P x) -> (sigT A P).

Section projections_sigT.

  Variable A:Type.
  Variable P:A->Type.

  Definition projT1 (* : (A:Type)(P:A->Type)(sigT A P) -> A *)
              := [H:(sigT A P)]Cases H of (existT x _) => x end.
   
  Definition projT2 (* : (A:Type)(P:A->Type)(p:(sigT A P))(P (projT1 A P p)) *)
              := [H:(sigT A P)]<[H:(sigT A P)](P (projT1 H))> 
                     Cases H of (existT x h) => h end.

End projections_sigT.

(* Pretty-printing of things in Specif.v *)

Syntax constr
  (* Default pretty-printing rules *)
  level 10:
    sig_var
      [(ABSTR_B_NB $c1 $c2)] -> [ [<hov 0> "sig " $c1:L [1 1] $c2:L ] ]
  | sig2_var
      [(Sig2_ABSTR_B_NB $c1 $c2)] -> [ [<hov 0> "sig2 " $c1:L [1 0] $c2:L ] ]
  | sigS_var
      [(SigS_ABSTR_B_NB $c1 $c2)] -> [ [<hov 0> "sigS " $c1:L [1 1] $c2:L ] ]
  | sigS2_var [(SigS2_ABSTR_B_NB $c1 $c2 $c3)]
      -> [ [<hov 0> "sigS2 " $c1:L [1 1] $c2:L [1 1] $c3:L] ]
  ;

  level 1:
(* Pretty-printing of [sig] *)
    sig  [<<(sig $c1 $c2)>>] -> [ (ABSTR_B_NB  $c1 $c2):E ]
  | sig_nb [(ABSTR_B_NB $c1 (LAMBDA $c1 [<>]$c2))]
      -> [ [<hov 0> "{_:" $c1:E " | " [0 1] $c2:E "}" ] ]
  | sigma_b [(ABSTR_B_NB $c1 (LAMBDA $c1 [$id]$c2))]
      -> [ [<hov 0> "{" $id ":" $c1:E " | " [0 1] $c2:E "}" ] ]

(* Pretty-printing of [sig2] *)
  | sig2 [<<(sig2 $c1 $c2 $c3)>>] -> [ (Sig2_ABSTR_B_NB  $c1 $c2 $c3):E ]
  | sig2_b_b
      [(Sig2_ABSTR_B_NB $c1 (LAMBDA $c1  [$id]$c2)  (LAMBDA $c1 [$id]$c3))]
       -> [ [<hov 0> "{"$id":"$c1:E"| " [0 1]$c2:E" & "$c3:E "}" ] ]
  | sig2_nb_b
      [(Sig2_ABSTR_B_NB $c1 (LAMBDA $c1 [<>]$c2) (LAMBDA  $c1 [$id]$c3))]
       -> [ [<hov 0> "{"$id":"$c1:E"| " [0 1]$c2:E" & "$c3:E "}" ] ]
  | sig2_b_nb
      [(Sig2_ABSTR_B_NB $c1 (LAMBDA $c1 [$id]$c2) (LAMBDA $c1 [<>]$c3))]
       -> [ [<hov 0> "{"$id":"$c1:E"| " [0 1]$c2:E" & "$c3:E "}" ] ]
  | sig2_nb_nb
      [(Sig2_ABSTR_B_NB $c1 (LAMBDA $c1 [<>]$c2) (LAMBDA $c1 [<>]$c3))]
       -> [ [<hov 0> "{_:"$c1:E "| " [0 1] $c2:E  " & " $c3:E "}" ] ]

(* Pretty-printing of [sigS] *)
  | sigS [<<(sigS $c1 $c2)>>] -> [(SigS_ABSTR_B_NB  $c1 $c2):E]
  | sigS_nb [(SigS_ABSTR_B_NB $c1 (LAMBDA $c1 [<>]$c2))]
       -> [ [<hov 0> "{_:" $c1:E [0 1]  " & " $c2:E  "}" ] ]
  | sigS_b [(SigS_ABSTR_B_NB $c1 (LAMBDA $c1 [$id]$c2))]
       -> [ [<hov 0> "{" $id  ":" $c1:E [0 1] " & " $c2:E "}" ] ]

(* Pretty-printing of [sigS2] *)
  | sigS2 [<<(sigS2 $c1 $c2 $c3)>>] -> [(SigS2_ABSTR_B_NB  $c1 $c2 $c3):E]
  | sigS2_b_b
      [(SigS2_ABSTR_B_NB $c1 (LAMBDA $c1  [$id]$c2)  (LAMBDA $c1 [$id]$c3))]
       -> [ [<hov 0> "{"$id ":" $c1:E [0 1]" & "$c2:E" & "$c3:E "}" ] ]
  | sigS2_nb_b
      [(SigS2_ABSTR_B_NB $c1 (LAMBDA $c1 [<>]$c2) (LAMBDA  $c1 [$id]$c3))]
       -> [ [<hov 0> "{"$id ":" $c1:E [0 1]" & "$c2:E" & "$c3:E "}" ] ]
  | sigS2_b_nb
      [ (SigS2_ABSTR_B_NB $c1 (LAMBDA $c1 [$id]$c2) (LAMBDA $c1 [<>]$c3))]
       -> [ [<hov 0> "{"$id ":" $c1:E [0 1]" & "$c2:E" & "$c3:E "}" ] ]
  | sigS2_nb_nb
      [(SigS2_ABSTR_B_NB $c1 (LAMBDA $c1 [<>]$c2) (LAMBDA $c1 [<>]$c3))]
       -> [ [<hov 0> "{_:"$c1:E [0 1]" & "$c2:E" & "$c3:E "}" ] ]

(* Pretty-printing of [projS1] and [projS2] *)
  | projS1_imp [<<(projS1 ? ? $a)>>] -> ["(ProjS1 " $a:E ")"]  
  | projS2_imp [<<(projS2 ? ? $a)>>] -> ["(ProjS2 " $a:E ")"]
  ;

(* Pretty-printing of [sumbool] and [sumor] *)
  level 4:
    sumbool [<<(sumbool $t1 $t2)>>]
       -> [ [<hov 0> "{" $t1:E "}" [0 1] "+" "{" $t2:L "}"] ]
  | sumor [<<(sumor $t1 $t2)>>]
       -> [ [<hov 0> $t1:E [0 1]  "+" "{" $t2:L "}"] ]
  ;

(* Pretty-printing of [except] *)
  level 1:
    Except_imp [<<(except $1 $t2)>>] -> [ [<hov 0> "Except " $t2 ] ]

(* Pretty-printing of [error] and [value] *)
  | Error_imp [<<(error $1 $t2)>>] -> [ [<hov 0> "Error " $t2 ] ]
  | Value_imp [<<(value $1 $t2)>>] -> [ [<hov 0> "Value " $t2 ] ].


(* $Id: Specif.v,v 1.10 1997/11/25 14:27:00 barras Exp $ *)
