(****************************************************************************)
(*                 The Calculus of Inductive Constructions                  *)
(*                                                                          *)
(*                                Projet Coq                                *)
(*                                                                          *)
(*                     INRIA                        ENS-CNRS                *)
(*              Rocquencourt                        Lyon                    *)
(*                                                                          *)
(*                                 Coq V6.1                                 *)
(*                               Oct 1st 1996                               *)
(*                                                                          *)
(****************************************************************************)
(*                                Streams.v                                 *)
(****************************************************************************)

Section Streams. (* The set of streams : definition *)

Variable A : Set.

CoInductive Set Stream   := Cons : A->Stream->Stream.


Definition head  := 
  [x:Stream] Cases x of (Cons a _) => a end.

Definition tail  := 
  [x:Stream] Cases x of (Cons _ s) => s end.

Fixpoint nth [n:nat] : Stream->A := 
  [s:Stream] Cases n of
	          O    => (head s) 
		|(S m) => (nth m (tail s))
	    end.


Lemma unfold_Stream :(x:Stream)x=(Cases x of (Cons a s) => (Cons a s) end). 
Intro x.
Case x.
Trivial.
Qed.


(* Extensional Equality between two streams  *)

CoInductive EqSt  : Stream->Stream->Prop := 
	    eqst : (s1,s2:Stream)
		   ((head s1)=(head s2))->
		   (EqSt (tail s1) (tail s2))
                    ->(EqSt s1 s2).

(* A coinduction principle *)

Tactic Definition CoInduction [$proof] := 
       [ <:tactic:<( Cofix $proof; (Intros; (Constructor;
                    [Clear $proof | Try (Apply $proof;Clear $proof)])))>> ].


(* Extensional equality is an equivalence relation *)


Theorem  EqSt_reflex : (s:Stream)(EqSt s s).
CoInduction EqSt_reflex.
Reflexivity.
Qed.

Theorem sym_EqSt : 
 (s1:Stream)(s2:Stream)(EqSt s1 s2)->(EqSt s2 s1).
CoInduction Eq_sym.
(Case H;Intros;Symmetry;Assumption).
(Case H;Intros;Assumption).
Qed.


Theorem trans_EqSt : 
 (s1,s2,s3:Stream)(EqSt s1 s2)->(EqSt s2 s3)->(EqSt s1 s3).
CoInduction Eq_trans.
Transitivity (head s2).
(Case H; Intros; Assumption).
(Case H0; Intros; Assumption).
Apply (Eq_trans (tail s1) (tail s2) (tail s3)).
(Case H;  Trivial).
(Case H0; Trivial).
Qed.

(* 
The definition given is equivalent to require the elements at each position to be equal 
*)

Theorem eqst_ntheq :
  (n:nat)(s1,s2:Stream)(EqSt s1 s2)->(nth n s1)=(nth n s2).
Induction n.
(Intros s1 s2 H; Case H; Trivial).
Intros m hypind.
Simpl.
Intros s1 s2 H.
Apply hypind.
(Case H; Trivial).
Qed.

Theorem ntheq_eqst : 
  (s1,s2:Stream)((n:nat)(nth n s1)=(nth n s2))->(EqSt s1 s2).
CoInduction Equiv2.
Apply (H O).
(Intros n; Apply (H (S n))).
Qed.

Section Stream_Properties.

Variable P : Stream->Prop.

Inductive   Exists :  Stream -> Prop :=
   Here    : (x:Stream)(P x) ->(Exists x) |
   Further : (x:Stream)~(P x)->(Exists (tail x))->(Exists x).

CoInductive ForAll : Stream  -> Prop :=
    forall : (x:Stream)(P x)->(ForAll (tail x))->(ForAll x).

End Stream_Properties.

End Streams.

Syntactic Definition cons := (Cons ?).
Syntactic Definition hd   := (head ?).
Syntactic Definition tl   := (tail ?).

Section Map.

Variables A,B : Set.
Variable  f   : A->B.

CoFixpoint map : (Stream A)->(Stream B) :=
 [s:(Stream A)](cons (f (hd s)) (map (tl s))).

End Map.


(* $Id: Streams.v,v 1.6 1998/01/24 06:14:40 herbelin Exp $ *)



