(****************************************************************************)
(*                 The Calculus of Inductive Constructions                  *)
(*                                                                          *)
(*                                Projet Coq                                *)
(*                                                                          *)
(*                     INRIA                        ENS-CNRS                *)
(*              Rocquencourt                        Lyon                    *)
(*                                                                          *)
(*                                 Coq V6.1                                 *)
(*                               Oct 1st 1996                               *)
(*                                                                          *)
(****************************************************************************)
(*                                 Zsyntax.v                                *)
(****************************************************************************)

Require Export fast_integer.
Require Export zarith_aux. 
Declare ML Module "g_zsyntax".

Grammar znatural ident :=
  nat_id [ prim:var($id) ] -> [$id]

with number :=

with formula :=
  form_expr [ expr($p) ] -> [$p]
| form_eq [ expr($p) "=" expr($c) ] -> [<<(eq Z $p $c)>>]
| form_le [ expr($p) "<=" expr($c) ] -> [<<(Zle $p $c)>>]
| form_lt [ expr($p) "<" expr($c) ] -> [<<(Zlt $p $c)>>]
| form_ge [ expr($p) ">=" expr($c) ] -> [<<(Zge $p $c)>>]
| form_gt [ expr($p) ">" expr($c) ] -> [<<(Zgt $p $c)>>]
| form_le_le [ expr($p) "<=" expr($c) "<=" expr($c1) ]
              -> [<<(Zle $p $c)/\(Zle $c $c1)>>]
| form_le_lt [ expr($p) "<=" expr($c) "<" expr($c1) ]
              -> [<<(Zle $p $c)/\(Zlt $c $c1)>>]
| form_lt_le [ expr($p) "<" expr($c) "<=" expr($c1) ]
              -> [<<(Zlt $p $c)/\(Zle $c $c1)>>]
| form_lt_lt [ expr($p) "<" expr($c) "<" expr($c1) ]
              -> [<<(Zlt $p $c)/\(Zlt $c $c1)>>]
| form_neq  [ expr($p) "<>" expr($c) ] -> [<<~(eq Z $p $c)>>]
| form_comp [ expr($p) "?" "=" expr($c) ] -> [<<(Zcompare $p $c)>>]

with expr :=
LEFTA
  expr_plus [ expr($p) "+" expr($c) ] -> [<<(Zplus $p $c)>>]
| expr_minus [ expr($p) "-" expr($c) ] -> [<<(Zminus $p $c)>>]
| expr2 [ expr2($e) ] -> [$e]
with expr2 :=
LEFTA
  expr_mult [ expr2($p) "*" expr2($c) ] -> [<<(Zmult $p $c)>>]
| expr1 [ expr1($e) ] -> [$e]
with expr1 :=
  expr_abs [ "|" expr($c) "|" ] -> [<<(Zabs $c)>>]
| expr_inv [ "-" expr1($c) ] -> [<<(Zinv $c)>>]
| expr0 [ expr0($e) ] -> [$e]
with expr0 :=
  expr_id [ ident($c) ] -> [$c]
| expr_com [ "[" command:command($c) "]" ] -> [$c]
| expr_appl [ "(" application($a) ")" ] -> [$a]
| expr_num [ number($s) ] -> [$s ]
  
with application :=
LEFTA
  apply [ application($p) application($c1) ] -> [<<($p $c1)>>]
| pair [ application($p) "," expr($c) ] -> [<<($p,$c)>>]
| appl0 [ application0($a) ] -> [$a]
with application0 :=
  appl_expr [ expr($c1) ] -> [$c1].

Grammar command command0 :=
  nat_in_com [ "[|" znatural:formula($c) "|]" ] -> [$c].

(* The symbols "[|" "|]" must be printed just once at the top of the expession,
  to avoid printings like | [| [|x|] + [|y|] |] < [|45|]| 
  for |x + y < 45|.
  So when a Z-expression is to be printed, its sub-expresssions are
  enclosed into an ast (ZEXPR $subexpr). (ZEXPR $s) is printed like $s
  but without symbols "[|" "|]" around. *)

(* There is just one problem: NEG and Zinv have the same printing rules.
   If Zinv is opaque, we may not be able to solve a goal like
    | -5 = -5| by reflexivity. (In fact, this precise Goal is solved
  by the Reflexivity tactic, but more complex problems may arise *)

Syntax constr
  level 0:
    Zle [<<(Zle $n1 $n2)>>] -> 
      [[<hov 0> "[|" (ZEXPR $n1) " <= " (ZEXPR $n2) "|]"]]
  | Zlt [<<(Zlt $n1 $n2)>>] -> 
      [[<hov 0> "[|"(ZEXPR $n1)" < "(ZEXPR $n2) "|]" ]]
  | Zge [<<(Zge $n1 $n2)>>] -> 
      [[<hov 0> "[|"(ZEXPR $n1)" >= "(ZEXPR $n2) "|]" ]]
  | Zgt [<<(Zgt $n1 $n2)>>] -> 
      [[<hov 0> "[|"(ZEXPR $n1)" > "(ZEXPR $n2) "|]" ]]
  | Zcompare [<<(Zcompare $n1 $n2)>>] ->
      [[<hov 0> "[|" (ZEXPR $n1) " ?= " (ZEXPR $n2) "|]" ]]
  | Zeq [<<(eq Z $n1 $n2)>>] -> 
      [[<hov 0> "[|"(ZEXPR $n1)" = "(ZEXPR $n2)"|]"]]
  | Zneq [<<~(eq Z $n1 $n2)>>] ->
      [[<hov 0> "[|"(ZEXPR $n1)" <> "(ZEXPR $n2) "|]"]]
  | Zle_Zle [<<(Zle $n1 $n2)/\(Zle $n2 $n3)>>] ->
      [[<hov 0> "[|"(ZEXPR $n1)" <= "(ZEXPR $n2)" <= "(ZEXPR $n3) "|]"]]
  | Zle_Zlt [<<(Zle $n1 $n2)/\(Zlt $n2 $n3)>>] ->
      [[<hov 0> "[|"(ZEXPR $n1)" <= "(ZEXPR $n2)" < "(ZEXPR $n3) "|]"]]
  | Zlt_Zle [<<(Zlt $n1 $n2)/\(Zle $n2 $n3)>>] ->
      [[<hov 0> "[|"(ZEXPR $n1)" < "(ZEXPR $n2)" <= "(ZEXPR $n3) "|]"]]
  | Zlt_Zlt [<<(Zlt $n1 $n2)/\(Zlt $n2 $n3)>>] ->
      [[<hov 0> "[|"(ZEXPR $n1)" < "(ZEXPR $n2)" < "(ZEXPR $n3) "|]"]]
  | ZZero [<<ZERO>>] -> ["|0|"]
  | ZPos [<<(POS $r)>>] -> ["[|" $r:"positive_printer" "|]"]
  | ZNeg [<<(NEG $r)>>] -> ["[|" $r:"negative_printer" "|]"]
  ;

  level 8:
    Zplus [<<(Zplus $n1 $n2)>>]
      -> [ [<hov 0> "[|"(ZEXPR $n1):E "+"(ZEXPR $n2):L "|]"] ]
  | Zminus [<<(Zminus $n1 $n2)>>]
      -> [ [<hov 0> "[|"(ZEXPR $n1):E "-"(ZEXPR $n2):L "|]"] ]
  ;

  level 7:
    Zmult [<<(Zmult $n1 $n2)>>]
      -> [ [<hov 0> "[|"(ZEXPR $n1):E "*"(ZEXPR $n2):L "|]"] ]
  ;

  level 6:
    Zinv [<<(Zinv $n1)>>] -> [ [<hov 0> "[|" "-"(ZEXPR $n1):E "|]"] ]
  ;

  level 5:
    Zabs [<<(Zabs $n1)>>] -> [  [<hov 0> "[||"(ZEXPR $n1):E "||]"] ]
  ;

  level 0:
    escape_inside [(ZEXPR $r)] -> [ "[" $r:E "]" ]
  ;

  level 4:
    Zappl_inside [(ZEXPR (APPLIST $h ($LIST $t)))]
      -> [ [<hov 0> "("(ZEXPR $h):E [1 0] (APPLINSIDETAIL ($LIST $t)):E ")"] ]
  | Zappl_inside_tail [(APPLINSIDETAIL $h ($LIST $t))]
      -> [(ZEXPR $h):E [1 0] (APPLINSIDETAIL ($LIST $t)):E] 
  | Zappl_inside_one [(APPLINSIDETAIL $e)] ->[(ZEXPR $e):E]
  ;

  level 3:
    var_inside [(ZEXPR ($VAR $id))] -> [$id]
  | const_inside [(ZEXPR (CONST $c))] -> [(CONST $c)]
  | constructor_inside [(ZEXPR (MUTCONSTRUCT ($ID $c1) ($ID $c2) ($ID $c3)))]
      -> [ (MUTCONSTRUCT $c1 $c2 $c3) ]
  ;

  level 8:
    Zplus_inside
      [(ZEXPR <<(Zplus $n1 $n2)>>)] -> [ (ZEXPR $n1):E "+"(ZEXPR $n2):L ]
  | Zminus_inside
      [(ZEXPR <<(Zminus $n1 $n2)>>)] -> [ (ZEXPR $n1):E "-" (ZEXPR $n2):L ]
  ;

  level 7:
    Zmult_inside
      [(ZEXPR <<(Zmult $n1 $n2)>>)] -> [ (ZEXPR $n1):E "*" (ZEXPR $n2):L ]
  ;

  level 6:
    Zinv_inside [(ZEXPR <<(Zinv $n1)>>)] -> [ "-" (ZEXPR $n1):E ]
  ;

  level 5:
    Zabs_inside [(ZEXPR <<(Zabs $n1)>>)] -> [ "|" (ZEXPR $n1) "|"]
  ;

  level 2:
    ZZero_inside [(ZEXPR <<ZERO>>)] -> ["0"]
  | ZPos_inside [(ZEXPR <<(POS $p)>>)] -> [$p:"positive_printer"]
  | ZNeg_inside [(ZEXPR <<(NEG $p)>>)] -> [$p:"negative_printer"].



(* $Id: Zsyntax.v,v 1.9 1998/01/28 13:52:30 loiseleu Exp $ *)
